'''
title: gravitywalker
by: fabius astelix @ 2009-03-23
version:@ 2009-03-23
revisers:
summary: an example on how to navigate a room using GravityWalker directcontrol component
'''
if not '_P3DPOD_' in globals():
  from pandac.PandaModules import loadPrcFileData
  loadPrcFileData("", '''#
model-path $MAIN_DIR/data/models/
model-path $MAIN_DIR/data/fonts/
model-path $MAIN_DIR/data/shaders/
sound-path $MAIN_DIR/data/sounds/
texture-path $MAIN_DIR/data/textures/
particle-path $MAIN_DIR/data/particles/
  ''')
import direct.directbase.DirectStart
from direct.showbase.DirectObject import DirectObject
from direct.showbase.InputStateGlobal import inputState
from direct.actor.Actor import Actor
from direct.task import Task
from direct.controls.GravityWalker import GravityWalker

from pandac.PandaModules import BitMask32
from pandac.PandaModules import CollisionTraverser
from pandac.PandaModules import ActorNode
#======================================================================
#
class World( DirectObject ):
  def __init__( self ):
    #define the collision traverser - it will take care of the collisions but it will be managed by the GravityWalker component
    base.cTrav = CollisionTraverser( )
    base.cTrav.showCollisions( render )
    #set 2 masks, one for the floor contacts between the avatar model and the collision surfaces
    mask_floor = BitMask32.bit( 1 )
    mask_walls = BitMask32.bit( 2 )
    # load and prepare the collison models - they have been created from the whole model selecting and duplicating just the surfaces in the 3Deditor (blender in the specific)
    self.floor = loader.loadModel( 'room_coll_floor' )
    self.floor.reparentTo( render )
    self.floor.setCollideMask( mask_floor )
    self.floor.hide()
    #
    self.walls = loader.loadModel( 'room_coll_walls' )
    self.walls.reparentTo( render )
    self.walls.setCollideMask( mask_walls )
    self.walls.hide()
    #here's the room model loading - this is loaded after the surfaces for no other reason than to show up the colliders ABOVE the whole model programatically
    self.room = loader.loadModel( 'room' )
    self.room.reparentTo( render )
    #
    self.actorNodePath = render.attachNewNode( ActorNode( 'Avatar' ) )
    self.actor = Actor("panda", {"walk":"panda-walk"})
    self.actor.setHpr(180, 0,0)
    self.actor.setScale(.5, .5, .5)
    self.actor.reparentTo( self.actorNodePath )
    #instancing and setting up the GravityWalker module - it is pretty self-explanatory
    self.walker = GravityWalker()
    self.walker.setAvatar( self.actor )
    self.walker.setWalkSpeed(forward=5, jump=15, reverse=4, rotate=70)
    self.walker.setWallBitMask( mask_walls )
    self.walker.setFloorBitMask( mask_floor )
    self.walker.initializeCollisions( base.cTrav, self.actorNodePath,
      avatarRadius = 2.5, floorOffset = -.1, reach = 1.5
    )
    self.walker.enableAvatarControls( )
    self.walker.placeOnFloor( )
    #here is where to specify which key fires which roaming event
    inputState.watch( 'forward', 'arrow_up', 'arrow_up-up' )
    inputState.watch( 'reverse', 'arrow_down', 'arrow_down-up' )
    inputState.watch( 'turnLeft', 'arrow_left', 'arrow_left-up' )
    inputState.watch( 'turnRight', 'arrow_right', 'arrow_right-up' )
    inputState.watch( 'slideLeft', 'a', 'a-up' )
    inputState.watch( 'slideRight', 'd', 'd-up' )
    inputState.watch( 'jump', 'control', 'control-up' )
    #this is a plus just to show the colliders
    self.accept('c-up', self.toggle_vwcolliders)
    #
    self.actor.loop("walk")
    self.actor.enableBlend()
    #let's enable the main loop - see below for details
    base.taskMgr.add( self.update, 'update' )

    base.disableMouse()
    base.camera.setPos(-20,40,20)
  #--------------------------------------------------------------
  #
  def toggle_vwcolliders(self):
    if self.floor.isHidden():
      self.room.hide()
      self.floor.show()
      self.walls.show()
    else:
      self.room.show()
      self.floor.hide()
      self.walls.hide()
  #--------------------------------------------------------------
  #
  def update( self, task ):
    '''the main loop where to do stuff related to what happens inside the GravityWalker component. In the specific we make the camera to follow the panda and make the panda do a walk animation while there is movement detected by the component'''
    # this is mandatori to make collisions happens inside the GravityWalker
    self.walker.oneTimeCollide( )
    self.actor.setPlayRate(.2*self.walker.speed, "walk")
    base.camera.lookAt(self.actor)
    return Task.cont

#----------------------------------------------------------------
pea=World()
if __name__ == "__main__":
  if not '_P3DPOD_' in globals(): run()
